'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

import os
import math
import random

import pymunk as pm
from pymunk.vec2d import Vec2d

import pyglet
from pyglet import resource

WINDOW_WIDTH = 600
WINDOW_HEIGHT = 600

mousex, mousey = 0. , 0.
#npcs = []
#powerups = []
projectiles = []

player = None

pm.init_pymunk()
space = pm.Space()
space.gravity = Vec2d(0.0, 0.0)
space.damping = 0.9

#data_py = os.path.abspath(os.path.dirname(__file__))
#data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))
data_dir = 'data'


def filepath(filename):
	'''Determine the path to a file in the data directory.
	'''
	return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
	'''Open a file in the data directory.

	"mode" is passed as the second arg to open().
	'''
	return open(os.path.join(data_dir, filename), mode)

def load_anim(filename, rows, cols, playspeed=0.1, loop=True):
	file_seq = resource.image(filepath(filename))
	images = pyglet.image.ImageGrid(file_seq, rows, cols)
	images = images.get_texture_sequence()
	map(centre_image, images)
	if (loop == True):
		#randomise start point of animation
		random_start = random.randint(0, len(images)-1)
		images = images[random_start+1 : len(images)] + images[0 : random_start]
	animation = pyglet.image.Animation.from_image_sequence(images, playspeed, loop=loop)
	return animation
	
def vector_to_rotation( dx=0., dy=0. ):
	if (dy == 0 and dx >= 0):
		return 90
	elif (dy == 0):
		return 270
	dx = float(dx)
	dy = float(dy)
	degrees = math.atan( dx / dy ) / (math.pi / 180)
	
	if ( dy < 0 ):
		degrees += 180

	return degrees

def rotation_to_vector( rot, length=1. ):
	rot = clip(rot, 0, 360)
	radians = rot * (math.pi / 180);
	x = math.sin(radians) * length;
	y = math.cos(radians) * length;
	
	return [x, y];

def clip(amount, clipfrom=0., clipto=100.):
	range = clipto - clipfrom
	if ( amount < clipfrom ):
		amount += range
	if ( amount > clipto ):
		amount -= range
	return amount

def dist( fromx, fromy, tox, toy ):
	return math.sqrt( (tox - fromx)*(tox - fromx) + (toy - fromy)*(toy - fromy) )
	

def centre_image(img):
	img.anchor_x = img.width / 2
	img.anchor_y = img.height / 2