'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import math
import random
import copy
import pyglet
from pyglet import resource
from pyglet.window import key
from pyglet.window import mouse

import pymunk as pm
from pymunk.vec2d import Vec2d

import data
import base
import levels
import player
import monsters
import projectiles
import powerups


window = pyglet.window.Window(data.WINDOW_WIDTH, data.WINDOW_HEIGHT)
keys = key.KeyStateHandler()
window.push_handlers(keys)

cursor_image = resource.image(data.filepath('cursor.png'))
cursor = pyglet.window.ImageMouseCursor(cursor_image, 8, 8)
window.set_mouse_cursor(cursor)

def main():
	load()
	pyglet.clock.schedule_interval(update, 1/60.)
	pyglet.app.run()

def load():
	data.player = player.Player()
	data.player.restore()
	window.push_handlers(data.player)
	
	levels.level_manager.switch_level()

def update(dt):
	data.space.step(dt)
	data.player.update(dt)
	levels.level_manager.update(dt)

@window.event
def on_draw():
	window.clear()
	levels.level_manager.draw()

@window.event
def on_key_press(symbol, modifiers):
	if (symbol == key._1):
		levels.level_manager.switch_level(0)
	if (symbol == key._2):
		levels.level_manager.switch_level(1)
	if (symbol == key._3):
		levels.level_manager.switch_level(2)

@window.event
def on_mouse_motion(x, y, dx, dy):
	data.mousex = x
	data.mousey = y

@window.event
def on_mouse_drag(x, y, dx, dy, buttons, modifiers):
	data.mousex = x
	data.mousey = y

@window.event
def on_mouse_press(x, y, button, modifiers):
	if button == mouse.LEFT:
		data.player.shoot()
		data.player.is_shooting = True

@window.event
def on_mouse_release(x, y, button, modifiers):
	if button == mouse.LEFT:
		data.player.is_shooting = False

@window.event
def on_mouse_scroll(x, y, scroll_x, scroll_y):
	#change weapon
	data.player.change_weapon( scroll_y )
	pass

def collision_func(shapeA, shapeB, contacts, normal_coef, data):
	#print "collision between " + str(shapeA.data) + " and " + str(shapeB.data)
	collide_types = ["projectile", "explosion", "powerup", "portal"]
	shapea_return = True
	shapeb_return = True
	# if this is a projectile collision
	if (shapeA.data.collide_type in collide_types):
		shapea_return = shapeA.data.collide( shapeB.data )
		
	if (shapeB.data.collide_type in collide_types):
		shapeb_return = shapeB.data.collide( shapeA.data )
	
	return (shapea_return and shapeb_return)

data.space.set_default_collisionpair_func(collision_func)

