import pyglet
from pyglet import resource

import data
import base
import weapons

class Monster(base.Character):
	mass = 75.
	move_speed = 500.
	
	rotate_speed = 100.
	anger = 10.
	quickturn = True
	weapon = None
	
	corpse = None
	corpse_batch = None
	
	def __init__(self, img, batch=None, corpse_batch=None):
		if (corpse_batch != None):
			self.corpse_batch = corpse_batch
		else:
			self.corpse_batch = batch
		super(Monster, self).__init__(img, batch=batch)
	
	def update(self, dt):
		self.weapon.update(dt)
		
		# behave according to our mood
		if ( self.anger > 0 ):
			if ( self.quickturn ):
				self.movetowards( data.player.x, data.player.y, self.move_speed * dt )
			else:
				self.rotatetowards( data.player.x, data.player.y, self.rotate_speed * dt )
		
		player_distance = data.dist( data.player.x, data.player.y, self.x, self.y )
		if ( player_distance < self.weapon.effective_range ):
			# attack the player
			self.weapon.fire(self.x, self.y, data.player.x, data.player.y)
		
		super(Monster, self).update(dt)

	def die(self):
		if ( self.corpse != None ):
			# create a corpse
			new_corpse = self.corpse( self.corpse_batch )
			new_corpse.x, new_corpse.y = self.x, self.y
			new_corpse.rotation = self.rotation
		super(Monster, self).die()



class ZombieCorpse(base.CollideSprite):
	def __init__(self, batch=None):
		img = resource.image(data.filepath('zombie_corpse.png'))
		super(ZombieCorpse, self).__init__(img, batch=batch)
		
class SkeletonCorpse(base.CollideSprite):
	def __init__(self, batch=None):
		img = resource.image(data.filepath('skeleton_corpse.png'))
		super(SkeletonCorpse, self).__init__(img, batch=batch)
		
class WormCorpse(base.CollideSprite):
	def __init__(self, batch=None):
		img = resource.image(data.filepath('worm_corpse.png'))
		super(WormCorpse, self).__init__(img, batch=batch)



class Zombie(Monster):
	move_speed = 300.
	rotate_speed = 50.
	quickturn = False
	
	corpse = ZombieCorpse
	
	def __init__(self, batch=None, corpse_batch=None):
		img = data.load_anim('zombie.png', 2, 8, 0.1, True)
		super(Zombie, self).__init__(img, batch=batch, corpse_batch=corpse_batch)
		self.reset()
		
	def reset(self):
		self.health = 100
		self.weapon = weapons.SingleRocket()
		self.weapon.parent_character = self

class Skeleton(Monster):
	move_speed = 300.
	rotate_speed = 50.
	quickturn = False
	
	corpse = SkeletonCorpse
	
	def __init__(self, batch=None, corpse_batch=None):
		img = data.load_anim('skeleton.png', 2, 8, 0.1, True)
		super(Skeleton, self).__init__(img, batch=batch, corpse_batch=corpse_batch)
		self.reset()
		
	def reset(self):
		self.health = 100
		self.weapon = weapons.Claw()
		self.weapon.parent_character = self
		

class Worm(Monster):
	move_speed = 200.
	rotate_speed = 50.
	quickturn = False
	
	corpse = WormCorpse
	
	def __init__(self, batch=None, corpse_batch=None):
		img = data.load_anim('worm.png', 1, 8, 0.1, True)
		super(Worm, self).__init__(img, batch=batch, corpse_batch=corpse_batch)
		self.reset()
		
	def reset(self):
		self.health = 10
		self.weapon = weapons.Chew()
		self.weapon.parent_character = self
		