import random

import pyglet
from pyglet import resource

import data
import projectiles
import levels

class Weapon(object):
	parent_character = None
	
	total_ammo = 30
	clip_size = 3
	round_delay = 0.1
	reload_delay = 1
	projectile_count = 1
	spread_degrees = 4
	projectile_type = projectiles.Bullet
	
	effective_range = 100.
	
	clip_ammo = 3
	wait = 0.
	
	def __init__(self):
		self.reset()
	
	def reset(self):
		self.clip_ammo = self.clip_size
	
	def update(self, dt):
		if ( self.wait > 0 ):
			self.wait -= dt
			if ( self.wait <= 0 ):
				self.wait = 0
				if ( self.clip_ammo == 0 ):
					if ( self.total_ammo > self.clip_size ):
						self.clip_ammo = self.clip_size
					else:
						self.clip_ammo = self.total_ammo
	
	def add_ammo(self, amount=10):
		self.total_ammo += amount
	
	def fire(self, x, y, tox=data.mousex, toy=data.mousey):
		#print "shooting, wait: " + str(self.wait) + " total_ammo: " + str(self.total_ammo) + " clip_ammo: " + str(self.clip_ammo)
		angle = data.vector_to_rotation( tox-x, toy-y )
		if ( self.wait == 0 and self.total_ammo > 0 ):
			for n in range(0, self.projectile_count):
				spread_offset = (random.random() * self.spread_degrees) - (self.spread_degrees * 0.5)
				new_proj = self.spawn_projectile()
				new_proj.fire(x, y, angle + spread_offset, tox, toy)
			
			self.clip_ammo -= 1
			self.total_ammo -= 1
			
			if (self.clip_ammo > 0):
				self.wait = self.round_delay
			else:
				self.wait = self.reload_delay
			
	def spawn_projectile(self):
		new_proj = levels.level_manager.current_level.add_projectile( self.projectile_type )
		new_proj.parent_weapon = self
		return new_proj


class Pistol(Weapon):
	clip_size = 1
	reload_delay = 0.3
	total_ammo = 100000
	
	effective_range = 400.
	
	def __init__(self):
		super(Pistol, self).__init__()
		self.reset()
		
	def reset(self):
		super(Pistol, self).reset()


class ScatterRocket(Weapon):
	clip_size = 1
	total_ammo = 100000
	reload_delay = 1
	projectile_count = 5
	spread_degrees = 90
	projectile_type = projectiles.HomingRocket
		
	effective_range = 300.
	
	def __init__(self):
		super(ScatterRocket, self).__init__()
		self.reset()
		
	def reset(self):
		super(ScatterRocket, self).reset()


class FlameThrower(Weapon):
	clip_size = 1
	total_ammo = 100000
	reload_delay = 0.1
	projectile_count = 1
	spread_degrees = 30
	projectile_type = projectiles.Flame
	
	effective_range = 90.
	
	def __init__(self):
		super(FlameThrower, self).__init__()
		self.reset()
		
	def reset(self):
		super(FlameThrower, self).reset()

class NuclearRocket(Weapon):
	clip_size = 1
	total_ammo = 100000
	reload_delay = 5
	projectile_count = 1
	spread_degrees = 30
	projectile_type = projectiles.NukeRocket

	effective_range = 300.
	
	def __init__(self):
		super(NuclearRocket, self).__init__()
		self.reset()
		
	def reset(self):
		super(NuclearRocket, self).reset()
		
#monster weapons
class Chew(Weapon):
	clip_size = 1
	total_ammo = 100000
	reload_delay = 0.1
	projectile_count = 1
	spread_degrees = 30
	projectile_type = projectiles.ChewProjectile
	
	effective_range = 25.
	
	def __init__(self):
		super(Chew, self).__init__()
		self.reset()
		
	def reset(self):
		super(Chew, self).reset()

class Claw(Weapon):
	clip_size = 1
	total_ammo = 100000
	reload_delay = 0.1
	projectile_count = 1
	spread_degrees = 30
	projectile_type = projectiles.ChewProjectile
	
	effective_range = 35.
	
	def __init__(self):
		super(Claw, self).__init__()

class SingleRocket(Weapon):
	clip_size = 1
	total_ammo = 100000
	reload_delay = 1
	projectile_count = 1
	spread_degrees = 90
	projectile_type = projectiles.HomingRocket

	effective_range = 300.
	
	def __init__(self):
		super(SingleRocket, self).__init__()
		self.reset()
		
	def reset(self):
		super(SingleRocket, self).reset()
